"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var generate_1 = require("./generate");
var default_1 = require("./default");
var util_1 = require("./util");
var BASE_TITLE_PREFIX = '###';
function generateMarkdown(file, config) {
    var lang = (config === null || config === void 0 ? void 0 : config.lang) || default_1.defaultLang;
    var markdownSchema = default_1.defaultMarkdownTableSchema[lang];
    if (!markdownSchema) {
        return;
    }
    var schemas = (0, generate_1.default)(file, config);
    if (!schemas) {
        return;
    }
    var getMarkdownTable = function (data, type) {
        var hasVersionTag = data.find(function (item) { var _a; return (_a = item === null || item === void 0 ? void 0 : item.tags) === null || _a === void 0 ? void 0 : _a.find(function (t) { return t.name === 'version'; }); });
        var tableColumns = [];
        markdownSchema.forEach(function (_a) {
            var title = _a.title, value = _a.value;
            title = typeof title === 'object' ? title[type] : title;
            value = typeof value === 'object' ? value[type] : value;
            if (hasVersionTag || value !== 'tag.version') {
                tableColumns.push({ title: title, value: value });
            }
        });
        var tableHeader = "|".concat(tableColumns.map(function (_a) {
            var title = _a.title;
            return title;
        }).join('|'), "|\n|").concat(tableColumns.map(function () { return '---'; }).join('|'), "|");
        var tableBody = data
            .map(function (schema) {
            var requiredTextWord = lang === 'zh' ? '必填' : 'Required';
            var requiredText = !schema.isOptional ? " **(".concat(requiredTextWord, ")**") : '';
            var singleLineMarkdown = tableColumns
                .map(function (column) {
                var _a;
                var field = column.value;
                // Field like tag.version
                var execResult = /tag\.(\w+)/.exec(field);
                if (execResult) {
                    field = execResult[1];
                    var obj = (_a = schema.tags) === null || _a === void 0 ? void 0 : _a.find(function (tag) { return tag.name === field; });
                    var value_1 = obj ? (0, util_1.toSingleLine)(obj.value) : '-';
                    return field === 'defaultValue' ? "`".concat(value_1, "`") : value_1;
                }
                var value = schema[field];
                switch (field) {
                    case 'type': {
                        return "`".concat(value, "`").concat(requiredText);
                    }
                    case 'initializerText': {
                        return value !== null ? "`".concat(value, "`") : '-';
                    }
                    default:
                        return value;
                }
            })
                .join('|');
            return "|".concat(singleLineMarkdown, "|");
        })
            .join('\n');
        return "".concat(tableHeader, "\n").concat(tableBody);
    };
    var getMarkdownFromSchema = function (title, schema) {
        var markdownTitle = "".concat(BASE_TITLE_PREFIX, " ").concat(title);
        var dataForTable = schema.data || schema.params;
        var tagMap = {};
        schema.tags.forEach(function (_a) {
            var name = _a.name, value = _a.value;
            return (tagMap[name] = value);
        });
        var description = tagMap[lang] || '';
        var table = dataForTable && dataForTable.length
            ? getMarkdownTable(dataForTable, schema.params ? 'parameter' : 'interface')
            : '';
        // Function type
        var _a = schema, params = _a.params, typeOfReturn = _a.returns;
        if (params) {
            var version = tagMap.version, returns = tagMap.returns;
            if (version) {
                description += "".concat(description ? '\n\n' : '').concat(BASE_TITLE_PREFIX, "# Since\n").concat(version);
            }
            description += "".concat(description ? '\n\n' : '').concat(BASE_TITLE_PREFIX, "# Returns\n`").concat(typeOfReturn, "`").concat(returns ? ": ".concat(returns) : '');
            table = "".concat(BASE_TITLE_PREFIX, "# Arguments\n").concat(table);
        }
        return [markdownTitle, description, table].filter(function (str) { return str; }).join('\n\n');
    };
    if (config === null || config === void 0 ? void 0 : config.strictDeclarationOrder) {
        var markdownList_1 = [];
        schemas.forEach(function (_a) {
            var title = _a.title, schema = _a.schema;
            return markdownList_1.push(getMarkdownFromSchema(title, schema));
        });
        return markdownList_1;
    }
    var markdownMap = {};
    Object.entries(schemas).forEach(function (_a) {
        var title = _a[0], schema = _a[1];
        markdownMap[title] = getMarkdownFromSchema(title, schema);
    });
    return markdownMap;
}
exports.default = generateMarkdown;
